package org.freecompany.util.text;

import org.freecompany.util.multicaster.AbstractMulticaster;

public class CharSequenceAggregate extends AbstractMulticaster< CharSequence> implements CharSequence {

	public static CharSequence NULL = new CharSequence() {
		public int length() { return 0; }
		public char charAt( int index) { throw new IndexOutOfBoundsException(); }
		public CharSequence subSequence( int start, int end) { throw new IndexOutOfBoundsException(); }
		public String toString() { return ""; }
	};

	public CharSequenceAggregate() {}
	public CharSequenceAggregate( final CharSequence a, final CharSequence b) {
		super( a, b);
	}

	/**
	 * Looks up the logical 'first' element in this aggregation.
	 */
	public CharSequence root() {
		return a instanceof CharSequenceAggregate ? (( CharSequenceAggregate) a).root() : a;
	}

	public int length() {
		return a.length() + b.length();
	}

	public char charAt( final int index) {
		return index >= a.length() ? b.charAt( index - a.length()) : a.charAt( index);
	}

	public CharSequence subSequence( int start, int end) {
		if ( end < a.length()) return a.subSequence( start, end);
		if ( start > a.length()) return b.subSequence( start - a.length(), end - a.length());
		return add( a.subSequence( start, a.length()), b.subSequence( 0, end - a.length()));
	}

	public String toString() {
		return new StringBuilder( a).append( b).toString();
	}

	public static CharSequence empty() {
		return NULL;
	}

	public static CharSequence add( CharSequence a, CharSequence b) {
		return AbstractMulticaster.add( CharSequenceAggregate.class, NULL, a, b);
	}

	public static CharSequence remove( CharSequence a, CharSequence b) {
		return AbstractMulticaster.remove( NULL, a, b);
	}
}
