package org.freecompany.util.text;

import java.util.*;

/**
 * A map implementation that can use <@link CharSequence> instances as keys.  It
 * also supports null keys and values.
 */
public class CharSequenceMap< T> extends TreeMap< CharSequence, T> {

	/**
	 * Comparator for examining two <@see java.lang.CharSequence> objects for equality based
	 * only on value.  This means the implementation is inconsistent with equals but allows
	 * for collections based on these objects which have undefined equals and hashcode methods
	 * naturally.
	 */
	public static class CharSequenceComparator implements Comparator< CharSequence> {
		public int compare( CharSequence one, CharSequence two) {
			if ( one == null) return two == null ? 0 : -1;
			if ( two == null) return 1;
			final int length = one.length();
			if ( length != two.length()) return one.length() - two.length();
			int comparison;
			for ( int x = 0; x < length; x++) if (( comparison = one.charAt( x) - two.charAt( x)) != 0) return comparison;
			return 0;
		}
	}

	public CharSequenceMap() {
		super( new CharSequenceComparator());
	}
}
