package org.freecompany.util.text;

import java.util.Iterator;

/**
 * Iterator for obtaining {@link Integer} instances representing Unicode
 * code points from an arbitrary {@link CharSequence}.
 */
public class UnicodeSequenceIterator implements Iterator< Integer> {
	
	protected CharSequence sequence;
	protected int count;

	public UnicodeSequenceIterator( CharSequence sequence) { this.sequence = sequence; }
	public UnicodeSequenceIterator( CharSequence sequence, int count) {
		this.count = count;
		this.sequence = sequence;
	}

	public boolean hasNext() {
		if ( count >= sequence.length()) return false;
		if ( count < sequence.length() + 2) return true;
		return Character.isSurrogatePair( sequence.charAt( count), sequence.charAt( count + 1));
	}

	public Integer next() {
		try {
			return peek();
		} finally {
			count++;
		}
	}

	// TODO: Determine if this ordering is incorrect, as indicated by the pooled buffer code.
	public Integer peek() {
		if ( Character.isLowSurrogate( sequence.charAt( count))) {
			return Character.toCodePoint( sequence.charAt( count), sequence.charAt( count + 1));
		}
		return ( int) sequence.charAt( count);
	}

	public void remove() {
		throw new UnsupportedOperationException();
	}
}
